@@ -568,8 +568,8 @@ impl std::fmt::Debug for Header<'_> {
 
 pub fn pkt_num_len(pn: u64, largest_acked: u64) -> usize {
     let num_unacked: u64 = pn.saturating_sub(largest_acked) + 1;
-    // computes ceil of num_unacked.log2()
-    let min_bits = u64::BITS - num_unacked.leading_zeros();
+    // computes ceil of num_unacked.log2() + 1
+    let min_bits = u64::BITS - num_unacked.leading_zeros() + 1;
     // get the num len in bytes
     min_bits.div_ceil(8) as usize
 }
@@ -1110,9 +1110,9 @@ impl PktNumManager {
         //
         // curr_pn
         //  |
-        //  v               |-lower-|
-        // [c x x x x x x x x x x x s s s s s s s s s s x x]
-        //    |--min_skip---|       |---skip_range----|
+        //  v                 |--- (upper - lower) ---|
+        // [c x x x x x x x x s s s s s s s s s s s s s x x]
+        //    |--min_skip---| |------skip_range-------|
         //
         //```
         let skip_pn_counter = MIN_SKIP_COUNTER_VALUE + lower + rand_skip_value;
@@ -1380,6 +1380,20 @@ mod tests {
         let hdr_num = u64::from(b.get_u24().unwrap());
         let pn = decode_pkt_num(0xace9fa, hdr_num, num_len);
         assert_eq!(pn, 0xace9fe);
+        // roundtrip
+        let base = 0xdeadbeef;
+        for i in 1..255 {
+            let pn = base + i;
+            let num_len = pkt_num_len(pn, base);
+            if num_len == 1 {
+                let decoded = decode_pkt_num(base, pn & 0xff, num_len);
+                assert_eq!(decoded, pn);
+            } else {
+                assert_eq!(num_len, 2);
+                let decoded = decode_pkt_num(base, pn & 0xffff, num_len);
+                assert_eq!(decoded, pn);
+            }
+        }
     }
 
     #[test]
