@@ -188,9 +188,9 @@ pub trait RecoveryOps {
     /// otherwise have constructed an ACK eliciting packet.
     fn should_elicit_ack(&self, epoch: packet::Epoch) -> bool;
 
-    fn get_acked_frames(&mut self, epoch: packet::Epoch) -> Vec<frame::Frame>;
+    fn next_acked_frame(&mut self, epoch: packet::Epoch) -> Option<frame::Frame>;
 
-    fn get_lost_frames(&mut self, epoch: packet::Epoch) -> Vec<frame::Frame>;
+    fn next_lost_frame(&mut self, epoch: packet::Epoch) -> Option<frame::Frame>;
 
     fn get_largest_acked_on_epoch(&self, epoch: packet::Epoch) -> Option<u64>;
     fn has_lost_frames(&self, epoch: packet::Epoch) -> bool;
@@ -265,7 +265,6 @@ pub trait RecoveryOps {
     #[cfg(test)]
     fn sent_packets_len(&self, epoch: packet::Epoch) -> usize;
 
-    #[cfg(test)]
     fn bytes_in_flight(&self) -> usize;
 
     fn bytes_in_flight_duration(&self) -> Duration;
@@ -366,10 +365,6 @@ pub enum CongestionControlAlgorithm {
     Reno            = 0,
     /// CUBIC congestion control algorithm (default). `cubic` in a string form.
     CUBIC           = 1,
-    /// BBR congestion control algorithm. `bbr` in a string form.
-    BBR             = 2,
-    /// BBRv2 congestion control algorithm. `bbr2` in a string form.
-    BBR2            = 3,
     /// BBRv2 congestion control algorithm implementation from gcongestion
     /// branch. `bbr2_gcongestion` in a string form.
     Bbr2Gcongestion = 4,
@@ -385,10 +380,7 @@ impl FromStr for CongestionControlAlgorithm {
         match name {
             "reno" => Ok(CongestionControlAlgorithm::Reno),
             "cubic" => Ok(CongestionControlAlgorithm::CUBIC),
-            "bbr" => Ok(CongestionControlAlgorithm::BBR),
-            #[cfg(not(feature = "gcongestion"))]
-            "bbr2" => Ok(CongestionControlAlgorithm::BBR2),
-            #[cfg(feature = "gcongestion")]
+            "bbr" => Ok(CongestionControlAlgorithm::Bbr2Gcongestion),
             "bbr2" => Ok(CongestionControlAlgorithm::Bbr2Gcongestion),
             "bbr2_gcongestion" => Ok(CongestionControlAlgorithm::Bbr2Gcongestion),
             _ => Err(crate::Error::CongestionControl),
@@ -712,7 +704,6 @@ pub enum StartupExitReason {
 mod tests {
     use super::*;
     use crate::packet;
-    use crate::recovery::congestion::PACING_MULTIPLIER;
     use crate::test_utils;
     use crate::CongestionControlAlgorithm;
     use crate::DEFAULT_INITIAL_RTT;
@@ -737,20 +728,12 @@ mod tests {
         assert!(!recovery_for_alg(algo).gcongestion_enabled());
 
         let algo = CongestionControlAlgorithm::from_str("bbr").unwrap();
-        assert_eq!(algo, CongestionControlAlgorithm::BBR);
-        assert!(!recovery_for_alg(algo).gcongestion_enabled());
+        assert_eq!(algo, CongestionControlAlgorithm::Bbr2Gcongestion);
+        assert!(recovery_for_alg(algo).gcongestion_enabled());
 
         let algo = CongestionControlAlgorithm::from_str("bbr2").unwrap();
-        #[cfg(not(feature = "gcongestion"))]
-        {
-            assert_eq!(algo, CongestionControlAlgorithm::BBR2);
-            assert!(!recovery_for_alg(algo).gcongestion_enabled());
-        }
-        #[cfg(feature = "gcongestion")]
-        {
-            assert_eq!(algo, CongestionControlAlgorithm::Bbr2Gcongestion);
-            assert!(recovery_for_alg(algo).gcongestion_enabled());
-        }
+        assert_eq!(algo, CongestionControlAlgorithm::Bbr2Gcongestion);
+        assert!(recovery_for_alg(algo).gcongestion_enabled());
 
         let algo =
             CongestionControlAlgorithm::from_str("bbr2_gcongestion").unwrap();
@@ -768,7 +751,7 @@ mod tests {
 
     #[rstest]
     fn loss_on_pto(
-        #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
+        #[values("reno", "cubic", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
@@ -1055,7 +1038,7 @@ mod tests {
 
     #[rstest]
     fn loss_on_timer(
-        #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
+        #[values("reno", "cubic", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
@@ -1252,7 +1235,7 @@ mod tests {
 
     #[rstest]
     fn loss_on_reordering(
-        #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
+        #[values("reno", "cubic", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
@@ -1739,7 +1722,7 @@ mod tests {
 
     #[rstest]
     fn pacing(
-        #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
+        #[values("reno", "cubic", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
@@ -1786,7 +1769,7 @@ mod tests {
         assert_eq!(r.bytes_in_flight_duration(), Duration::ZERO);
 
         // Next packet will be sent out immediately.
-        if cc_algorithm_name != "bbr2_gcongestion" {
+        if cc_algorithm_name == "cubic" || cc_algorithm_name == "reno" {
             assert_eq!(r.pacing_rate(), 0);
         } else {
             assert_eq!(r.pacing_rate(), 103963);
@@ -1861,8 +1844,8 @@ mod tests {
         assert_eq!(r.bytes_in_flight(), 6000);
         assert_eq!(r.bytes_in_flight_duration(), Duration::from_millis(50));
 
-        if cc_algorithm_name != "bbr2_gcongestion" {
-            // Pacing is not done during initial phase of connection.
+        if cc_algorithm_name == "cubic" || cc_algorithm_name == "reno" {
+            // Pacing is disabled.
             assert_eq!(r.get_packet_send_time(now), now);
         } else {
             // Pacing is done from the beginning.
@@ -1936,18 +1919,7 @@ mod tests {
         // We pace this outgoing packet. as all conditions for pacing
         // are passed.
         let pacing_rate = match cc_algorithm_name {
-            "bbr" => {
-                // Constants from congestion/bbr/mod.rs
-                let cwnd_gain = 2.0;
-                let startup_pacing_gain = 2.89;
-                // Adjust for cwnd_gain.  BW estimate was made before the CWND
-                // increase.
-                let bw = r.cwnd() as f64 /
-                    cwnd_gain /
-                    Duration::from_millis(50).as_secs_f64();
-                (bw * startup_pacing_gain) as u64
-            },
-            "bbr2_gcongestion" => {
+            "bbr2_gcongestion" | "bbr2" => {
                 let cwnd_gain: f64 = 2.0;
                 // Adjust for cwnd_gain.  BW estimate was made before the CWND
                 // increase.
@@ -1956,27 +1928,13 @@ mod tests {
                     Duration::from_millis(50).as_secs_f64();
                 bw as u64
             },
-            "bbr2" => {
-                // Constants from congestion/bbr2/mod.rs
-                let cwnd_gain = 2.0;
-                let startup_pacing_gain = 2.77;
-                let pacing_margin_percent = 0.01;
-                // Adjust for cwnd_gain.  BW estimate was made before the CWND
-                // increase.
-                let bw = r.cwnd() as f64 /
-                    cwnd_gain /
-                    Duration::from_millis(50).as_secs_f64();
-                (bw * startup_pacing_gain * (1.0 - pacing_margin_percent)) as u64
-            },
-            _ => {
-                let bw =
-                    r.cwnd() as f64 / Duration::from_millis(50).as_secs_f64();
-                (bw * PACING_MULTIPLIER) as u64
-            },
+            _ => 0,
         };
         assert_eq!(r.pacing_rate(), pacing_rate);
 
-        let scale_factor = if cc_algorithm_name == "bbr2_gcongestion" {
+        let scale_factor = if cc_algorithm_name == "bbr2_gcongestion" ||
+            cc_algorithm_name == "bbr2"
+        {
             // For bbr2_gcongestion, send time is almost 13000 / pacing_rate.
             // Don't know where 13000 comes from.
             1.08333332
@@ -1985,7 +1943,15 @@ mod tests {
         };
         assert_eq!(
             r.get_packet_send_time(now) - now,
-            Duration::from_secs_f64(scale_factor * 12000.0 / pacing_rate as f64)
+            if cc_algorithm_name == "bbr2_gcongestion" ||
+                cc_algorithm_name == "bbr2"
+            {
+                Duration::from_secs_f64(
+                    scale_factor * 12000.0 / pacing_rate as f64,
+                )
+            } else {
+                Duration::ZERO
+            }
         );
         assert_eq!(r.startup_exit(), None);
     }
@@ -2178,7 +2144,7 @@ mod tests {
 
     #[rstest]
     fn pmtud_loss_on_timer(
-        #[values("reno", "cubic", "bbr", "bbr2", "bbr2_gcongestion")]
+        #[values("reno", "cubic", "bbr2", "bbr2_gcongestion")]
         cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
@@ -2325,11 +2291,7 @@ mod tests {
         assert_eq!(r.in_flight_count(packet::Epoch::Application), 0);
         assert_eq!(r.bytes_in_flight(), 0);
         assert_eq!(r.bytes_in_flight_duration(), Duration::from_micros(11250));
-        assert_eq!(r.cwnd(), match cc_algorithm_name {
-            "bbr" => 14000,
-            "bbr2" => 14000,
-            _ => 12000,
-        });
+        assert_eq!(r.cwnd(), 12000);
 
         assert_eq!(r.lost_count(), 0);
 
@@ -2353,7 +2315,7 @@ mod tests {
     // congestion specific algorithms.
     #[rstest]
     fn congestion_delivery_rate(
-        #[values("reno", "cubic", "bbr", "bbr2")] cc_algorithm_name: &str,
+        #[values("reno", "cubic", "bbr2")] cc_algorithm_name: &str,
     ) {
         let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
         assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
@@ -2410,6 +2372,156 @@ mod tests {
         );
         assert_eq!(r.startup_exit(), None);
     }
+
+    #[rstest]
+    fn acks_with_no_retransmittable_data(
+        #[values("reno", "cubic", "bbr2_gcongestion")] cc_algorithm_name: &str,
+    ) {
+        let rtt = Duration::from_millis(100);
+
+        let mut cfg = Config::new(crate::PROTOCOL_VERSION).unwrap();
+        assert_eq!(cfg.set_cc_algorithm_name(cc_algorithm_name), Ok(()));
+
+        let mut r = Recovery::new(&cfg);
+
+        let mut now = Instant::now();
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 0);
+
+        let mut next_packet = 0;
+        // send some packets.
+        for _ in 0..3 {
+            let p = test_utils::helper_packet_sent(next_packet, now, 1200);
+            next_packet += 1;
+            r.on_packet_sent(
+                p,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now,
+                "",
+            );
+        }
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 3);
+        assert_eq!(r.bytes_in_flight(), 3600);
+        assert_eq!(r.bytes_in_flight_duration(), Duration::ZERO);
+
+        assert_eq!(
+            r.pacing_rate(),
+            if cc_algorithm_name == "bbr2_gcongestion" {
+                103963
+            } else {
+                0
+            },
+        );
+        assert_eq!(r.get_packet_send_time(now), now);
+        assert_eq!(r.cwnd(), 12000);
+        assert_eq!(r.cwnd_available(), 8400);
+
+        // Wait 1 rtt for ACK.
+        now += rtt;
+
+        let mut acked = RangeSet::default();
+        acked.insert(0..3);
+
+        assert_eq!(
+            r.on_ack_received(
+                &acked,
+                10,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now,
+                None,
+                "",
+            )
+            .unwrap(),
+            OnAckReceivedOutcome {
+                lost_packets: 0,
+                lost_bytes: 0,
+                acked_bytes: 3600,
+                spurious_losses: 0,
+            }
+        );
+
+        assert_eq!(r.sent_packets_len(packet::Epoch::Application), 0);
+        assert_eq!(r.bytes_in_flight(), 0);
+        assert_eq!(r.bytes_in_flight_duration(), rtt);
+        assert_eq!(r.rtt(), rtt);
+
+        // Pacing rate is recalculated based on initial cwnd when the
+        // first RTT estimate is available.
+        assert_eq!(
+            r.pacing_rate(),
+            if cc_algorithm_name == "bbr2_gcongestion" {
+                120000
+            } else {
+                0
+            },
+        );
+
+        // Send some no "in_flight" packets
+        for iter in 3..1000 {
+            let mut p = test_utils::helper_packet_sent(next_packet, now, 1200);
+            // `in_flight = false` marks packets as if they only contained ACK
+            // frames.
+            p.in_flight = false;
+            next_packet += 1;
+            r.on_packet_sent(
+                p,
+                packet::Epoch::Application,
+                HandshakeStatus::default(),
+                now,
+                "",
+            );
+
+            now += rtt;
+
+            let mut acked = RangeSet::default();
+            acked.insert(iter..(iter + 1));
+
+            assert_eq!(
+                r.on_ack_received(
+                    &acked,
+                    10,
+                    packet::Epoch::Application,
+                    HandshakeStatus::default(),
+                    now,
+                    None,
+                    "",
+                )
+                .unwrap(),
+                OnAckReceivedOutcome {
+                    lost_packets: 0,
+                    lost_bytes: 0,
+                    acked_bytes: 0,
+                    spurious_losses: 0,
+                }
+            );
+
+            // Verify that connection has not exited startup.
+            assert_eq!(r.startup_exit(), None, "{iter}");
+
+            // Unchanged metrics.
+            assert_eq!(
+                r.sent_packets_len(packet::Epoch::Application),
+                0,
+                "{iter}"
+            );
+            assert_eq!(r.bytes_in_flight(), 0, "{iter}");
+            assert_eq!(r.bytes_in_flight_duration(), rtt, "{iter}");
+            assert_eq!(
+                r.pacing_rate(),
+                if cc_algorithm_name == "bbr2_gcongestion" ||
+                    cc_algorithm_name == "bbr2"
+                {
+                    120000
+                } else {
+                    0
+                },
+                "{iter}"
+            );
+        }
+    }
 }
 
 mod bandwidth;
